// 12bit A/D converter test program
// RB6(AN6) : A/D input
// RB8(OC1) : PWM output

#include <p30f2012.h>     // include dsPIC30F2012 header file

//===============================================================
// 1us delay

void wait(void) {
    asm("repeat #22");  // for 29.48MHz clock
    asm("nop");
}

//===============================================================
// t*10us delay

void delay10us(unsigned t) {
  unsigned i;

  for (i=0; i<t; i=i+1) {
    asm("repeat #284");  // for 29.48MHz clock
    asm("nop");
  }  
}

//===============================================================
// t*1ms delay

void delay1ms(unsigned t) {
  unsigned i, j;

  for (i=0; i<t; i=i+1) {
    for (j=0; j<100; j=j+1) {
      asm("repeat #286");  // for 29.48MHz clock
      asm("nop");
    }
  }  
}

//===============================================================

void __attribute__ ((__interrupt__)) _AltT3Interrupt(void) {
  PORTFbits.RF6=~PORTFbits.RF6;     // toggle RF6 output
  PR3=15*(4096-ADCBUF0)+2000;       // set tone period  
  IFS0bits.T3IF=0;                  // clear interrupt flag
}

//===============================================================

int main(void) {
  unsigned i;

  ADPCFG=0xffbf;                    // use RB6 for A/D input
  ADCON1=0x0006;

  ADCON2=0x0000;
  ADCON3=0x0010;                    // use interal RC clock
  ADCHS =0x0006;
  ADCSSL=0x0000;                    // no scan

  TRISBbits.TRISB8=0;               // RB8(OC1) is output

  T2CON  =0x0000;                   // without prescaler    
  PR2    =4096;                     // set Timer2 period register
  TMR2   =0x0000;                   // clear Timer2 counter register

  OC1RS =0;                         // set duty parameter
  OC1R  =0x0000;                    // clear OC1R
  OC1CON=0x0006;                    // use Timer2 for PWM

  TRISFbits.TRISF6 =0;              // RF6 is tone output
  T3CON=0x0000;                     // clear TIMER3 control register
  PR3  =6000;                       // set TIMER3 period register
  IFS0 =0x0000;                     // clear all interrupt
  INTCON1=0x0000;                   // set INTCON1
  INTCON2=0x8000;                   // use AIVT (Alternate IVT)
  IEC0 =0x0080;                     // enable TIMER3 interrupt
  T3CON=0x0000;                     // 

  T3CONbits.TON=1;
  T2CONbits.TON=1;                  // start Timer2

  ADCON1bits.ADON=1;                // A/D on
  wait();                           // wait until data hold

  while(1) {                        // forever loop
    ADCON1bits.SAMP=0;              // start A/D conversion
    while (!ADCON1bits.DONE) { ; }  // while conversion not done
    OC1RS=ADCBUF0;                  // read A/D data and set OC1RS
    wait();                         // wait....
  }

}
