// I2C EEPROM read/write test
//   24LC256
//     SDA=RF2 
//     SCL=RF3
//     A0=A1=A2=WP=0(GND)
//   RS-232C input
//     RC14=UARX
//   piezo sounder output
//     RB6

#include <p30f2012.h>     // include dsPIC30F2012 header file

//===============================================================

unsigned eeprom_read(unsigned adrs) {
  I2CCONbits.SEN=1;                       // start condition
  while (I2CCONbits.SEN==1) { ; }
  I2CTRN=0xa0;                            // send control byte
  while (I2CSTATbits.TRSTAT==1) { ; }
  I2CTRN=(adrs&0xff00)>>8;                // send address (high)
  while (I2CSTATbits.TRSTAT==1) { ; }
  I2CTRN=adrs&0x00ff;                     // send address (low)
  while (I2CSTATbits.TRSTAT==1) { ; }
  I2CCONbits.RSEN=1;                      // repeated start condition
  while (I2CCONbits.RSEN==1) { ; }
  I2CTRN=0xa1;                            // send control byte 
  while (I2CSTATbits.TRSTAT==1) { ; }
  I2CCONbits.RCEN=1;                      // read data byte
  while (I2CCONbits.RCEN==1) { ; }           
  I2CCONbits.ACKDT=1;                     // set ACK/NACK value  
  I2CCONbits.ACKEN=1;                     // send NACK
  while (I2CCONbits.ACKEN==1) { ; }
  I2CCONbits.PEN=1;                       // stop condition
  while (I2CCONbits.PEN==1) { ; }
  return I2CRCV;
}

//===============================================================

void eeprom_write(unsigned adrs, unsigned char data) {

  // data write sequence
  I2CCONbits.SEN=1;                       // start condition
  while (I2CCONbits.SEN==1) { ; }
  I2CTRN=0xa0;                            // send control byte
  while (I2CSTATbits.TRSTAT==1) { ; }
  I2CTRN=(adrs&0xff00)>>8;                // send address (high)
  while (I2CSTATbits.TRSTAT==1) { ; }
  I2CTRN=adrs&0x00ff;                     // send address (low)
  while (I2CSTATbits.TRSTAT==1) { ; }
  I2CTRN=data;                            // write data byte
  while (I2CSTATbits.TRSTAT==1) { ; }
  I2CCONbits.PEN=1;                       // stop condition
  while (I2CCONbits.PEN==1) { ; }     
  
  // acknowledge polling
  I2CCONbits.SEN=1;                       // start condition
  while (I2CCONbits.SEN==1) { ; }
  I2CTRN=0xa0;                            // send control byte
  while (I2CSTATbits.TRSTAT==1) { ; }
  while (I2CSTATbits.ACKSTAT==1) {
    asm("repeat #8000");  asm("nop");     // 300us delay (for 29.48MHz dsPIC)
    I2CCONbits.RSEN=1;                    // repeated start condition
    while (I2CCONbits.RSEN==1) { ; }
    I2CTRN=0xa0;                          // send control byte
    while (I2CSTATbits.TRSTAT==1) { ; }
  } 
  I2CCONbits.PEN=1;                       // stop condition
  while (I2CCONbits.PEN==1) { ; }

}

//===============================================================

int main(void) {
  unsigned adrs, data;
  unsigned i, t;

  U1MODE=0x8400;        // use U1ATX 
  U1BRG =95;            // 19200bps (29.48MHz/16/19200-1=95)
  TRISBbits.TRISB6=0;   // RB6 is output

  I2CBRG=0x180;         // about 70kHz I2C clock
  I2CCONbits.I2CEN=1;   // I2C enable

  adrs=0;
  t=1000;
  while(1) {
    for (i=0; i<t; i=i+1) asm("nop");
    PORTBbits.RB6=~PORTBbits.RB6;
    if (U1STAbits.URXDA==1) {
      data=U1RXREG;
      eeprom_write(adrs, data);
      data=eeprom_read(adrs);
      adrs=(adrs+1)%32768;
      t=600*(9-(data-'0')%10)+400;
    }
  }

}
