
#include <p30f2012.h>     // include dsPIC30F2012 header file

//===============================================================
// 1us delay

void wait(void) {
    asm("repeat #22");                // for 29.48MHz clock
    asm("nop");
}

//===============================================================
// t*10us delay

void delay10us(unsigned t) {
  unsigned i;

  for (i=0; i<t; i=i+1) {
    asm("repeat #284");               // for 29.48MHz clock
    asm("nop");
  }  
}

//===============================================================
// t*1ms delay

void delay1ms(unsigned t) {
  unsigned i, j;

  for (i=0; i<t; i=i+1) {
    for (j=0; j<100; j=j+1) {
      asm("repeat #286");             // for 29.48MHz clock
      asm("nop");
    }
  }  
}

//===============================================================

void lcd_tx4bit(unsigned rs, unsigned data) {
  rs=(rs&0x0001)<<4;                  // rs   --> RB4 
  data=data&0x000f;                   // data --> RB0-RB3
  PORTB=rs|data;                      // E=0, rs, data
  wait();                             // 1us delay
  PORTB=0x0020|PORTB;                 // E=1, rs, data
  wait();                             // 1us delay  
  PORTB=0x001f&PORTB;                 // E=0, rs, data
  delay10us(5);                       // 50us delay
}

//===============================================================

void lcd_tx8bit(unsigned rs, unsigned data) {
  unsigned hdata, ldata;

  hdata=(0x00f0&data)>>4;             // b7-b4 --> hdata
  ldata=0x000f&data;                  // b3-b0 --> ldata
  if (rs==0) {                        // if DDRAM address set
    lcd_tx4bit(0, 0x0008|hdata);      // DB7=1
    lcd_tx4bit(0, ldata);
  } else {                            // else DDRAM data write
    lcd_tx4bit(1, hdata);
    lcd_tx4bit(1, ldata);
  }
}

//===============================================================

void lcd_putchar(unsigned char c) {
  static unsigned xpos=0, ypos=0;  
  unsigned        i;

  if (c==0x00) {                      // if NULL then cursor goto home
    lcd_tx4bit(0, 0b0000);
    lcd_tx4bit(0, 0b0010);
    delay1ms(2);
    xpos=0;  ypos=0;  
  } else if (c==0x0b) {               // else if ESC then clear screen
    lcd_tx4bit(0, 0b0000);
    lcd_tx4bit(0, 0b0001);
    delay1ms(2);
    xpos=0;  ypos=0;
  } else {                            // else put character
    lcd_tx8bit(0, xpos+0x0040*ypos);
    lcd_tx8bit(1, c);     
    xpos=xpos+1;
    if (xpos==16) {
      xpos=0;
      ypos=(ypos+1)%2;
    }
  }
}

//===============================================================

void lcd_puthex(unsigned char c) {
  static char h[16]="0123456789ABCDEF";

  lcd_putchar(h[c/16]);
  lcd_putchar(h[c%16]);
  lcd_putchar(' ');
}

//===============================================================

int main(void) {
  unsigned i;

  //ADPCFG=0xffff;                    // disable all A/D input pin

  // LCD unit initialize
  TRISB=0xffc0;                       // RB0-RB5 is output
  PORTB=0x0000;                       // RB0-RB5 clear
  delay1ms(40);                       // 40ms delay
  lcd_tx4bit(0, 0b0011); 
  delay1ms(5);        
  lcd_tx4bit(0, 0b0011); 
  delay1ms(1);       
  lcd_tx4bit(0, 0b0011);  
  delay1ms(1);        
  lcd_tx4bit(0, 0b0010);  
  delay1ms(1); 
  lcd_tx4bit(0, 0b0010); 
  lcd_tx4bit(0, 0b1000); 
  lcd_tx4bit(0, 0b0000); 
  lcd_tx4bit(0, 0b1000);
  lcd_tx4bit(0, 0b0000);
  lcd_tx4bit(0, 0b0001);  
  delay1ms(2);
  lcd_tx4bit(0, 0b0000); 
  lcd_tx4bit(0, 0b0110); 

  // display on, cursor off, blink on
  lcd_tx4bit(0, 0b0000); 
  lcd_tx4bit(0, 0b1101); 
  delay1ms(1000);

  while(1) {
    for (i=0; i<256; i=i+1) {
      lcd_puthex(i);
      delay1ms(400);
    }
    lcd_putchar(0x00);
    for (i=0; i<64; i=i+1) {
      lcd_putchar(i+32);
      delay1ms(400);
    }
    lcd_putchar(0x0b);
    for (i=0; i<64; i=i+1) {
      lcd_putchar(i+0x00a0);
      delay1ms(400);
    }
  }

}
