// LM75 read test
//   SDA=RF2 
//   SCL=RF3
//   A0=A1=A2=0(GND)
// PWM output
//   RB8

#include <p30f2012.h>     // include dsPIC30F2012 header file

//===============================================================
// t*1ms delay

void delay1ms(unsigned t) {
  unsigned i, j;

  for (i=0; i<t; i=i+1) {
    for (j=0; j<100; j=j+1) {
      asm("repeat #286");               // for 29.48MHz clock
      asm("nop");
    }
  }  
}

//===============================================================

int lm75read() {
  int data;

  I2CCONbits.SEN=1;                     // start condition
  while (I2CCONbits.SEN==1) { ; }
  I2CTRN=0x91;                          // send control byte
  while (I2CSTATbits.TRSTAT==1) { ; } 
  I2CCONbits.RCEN=1;                    // read data byte (high)
  while (I2CCONbits.RCEN==1) { ; }
  I2CCONbits.ACKDT=0;                   // set ACK/NACK value
  I2CCONbits.ACKEN=1;                   // send ACK
  while (I2CCONbits.ACKEN==1) { ; }
  data=I2CRCV<<1;
  I2CCONbits.RCEN=1;                    // read data byte (low)
  while (I2CCONbits.RCEN==1) { ; }
  I2CCONbits.ACKDT=1;                   // set ACK/NACK value
  I2CCONbits.ACKEN=1;                   // send NACK
  while (I2CCONbits.ACKEN==1) { ; }
  data=data|(I2CRCV>>7);
  I2CCONbits.PEN=1;                     // stop condition
  while (I2CCONbits.PEN==1) { ; }
  return data;
}

//===============================================================

int main(void) {
  int temp;

  I2CBRG=0x180;                         // about 70kHz I2C clock
  I2CCONbits.I2CEN=1;                   // I2C enable

  TRISBbits.TRISB8=0;                   // RB8(OC1) is output
  T2CON  =0x0000;                       // without prescaler    
  PR2    =1000;                         // set Timer2 period register
  TMR2   =0x0000;                       // clear Timer2 counter register
  OC1RS =0;                             // set duty parameter
  OC1R  =0x0000;                        // clear OC1R
  OC1CON=0x0006;                        // use Timer2 for PWM
  T2CONbits.TON=1;                      // start Timer2

  while(1) {
    temp=lm75read();                    // read LM75 output
    OC1RS=10*temp;                      // 0C=0V, 50C=5V
    delay1ms(500);                      // LM75 requires conversion time greater than 300ms
  }

}
