
#include <p30f2012.h>     // include dsPIC30F2012 header file

volatile unsigned mouse_data[3]={0, 0, 0};

//===============================================================
// 1us delay

void wait(void) {
    asm("repeat #22");                  // for 29.48MHz clock
    asm("nop");
}

//===============================================================
// t*10us delay

void delay10us(unsigned t) {
  unsigned i;

  for (i=0; i<t; i=i+1) {
    asm("repeat #284");                 // for 29.48MHz clock
    asm("nop");
  }  
}

//===============================================================
// t*1ms delay

void delay_ms(unsigned t) {
  unsigned i, j;

  for (i=0; i<t; i=i+1) {
    for (j=0; j<100; j=j+1) {
      asm("repeat #286");               // for 29.48MHz clock
      asm("nop");
    }
  }  
}

//===============================================================

void lcd_tx4bit(unsigned rs, unsigned data) {
  rs=(rs&0x0001)<<4;                    // rs   --> RB4 
  data=data&0x000f;                     // data --> RB0-RB3
  PORTB=rs|data;                        // E=0, rs, data
  wait();                               // 1us delay
  PORTB=0x0020|PORTB;                   // E=1, rs, data
  wait();                               // 1us delay  
  PORTB=0x001f&PORTB;                   // E=0, rs, data
  delay10us(5);                         // 50us delay
}

//===============================================================

void lcd_tx8bit(unsigned rs, unsigned data) {
  unsigned hdata, ldata;

  hdata=(0x00f0&data)>>4;               // b7-b4 --> hdata
  ldata=0x000f&data;                    // b3-b0 --> ldata
  if (rs==0) {                          // if DDRAM address set
    lcd_tx4bit(0, 0x0008|hdata);        // DB7=1
    lcd_tx4bit(0, ldata);
  } else {                              // else DDRAM data write
    lcd_tx4bit(1, hdata);
    lcd_tx4bit(1, ldata);
  }
}

//===============================================================

void lcd_putchar(unsigned char c) {
  static unsigned xpos=0, ypos=0;  
  unsigned        i;

  if (c==0x00) {                        // if NULL then cursor goto home
    lcd_tx4bit(0, 0b0000);
    lcd_tx4bit(0, 0b0010);
    delay_ms(2);
    xpos=0;  ypos=0;  
  } else if (c==0x0b) {                 // else if ESC then clear screen
    lcd_tx4bit(0, 0b0000);
    lcd_tx4bit(0, 0b0001);
    delay_ms(2);
    xpos=0;  ypos=0;
  } else {                              // else put character
    lcd_tx8bit(0, xpos+0x0040*ypos);
    lcd_tx8bit(1, c);     
    xpos=xpos+1;
    if (xpos==16) {
      xpos=0;
      ypos=(ypos+1)%2;
    }
  }
}

//===============================================================

void lcd_puthex(unsigned char c) {
  static char h[16]="0123456789ABCDEF";

  lcd_putchar(h[c/16]);
  lcd_putchar(h[c%16]);
  lcd_putchar(' ');
}

//===============================================================      

void __attribute__ ((__interrupt__)) _AltINT0Interrupt(void) {
  unsigned i, p;
  unsigned weight, val;

  PORTCbits.RC13=~PORTCbits.RC13;       // toggle RC13 output (LED) 
  for (p=0; p<3; p=p+1) {
    val=0;
    weight=1;
    while (PORTFbits.RF6==1) { ; }      // clock=1
    while (PORTFbits.RF6==0) { ; }      // start bit
    while (PORTFbits.RF6==1) { ; }      // b0
    for (i=0; i<8; i=i+1) {
      val=val+PORTFbits.RF5*weight;
      weight=2*weight;
      while (PORTFbits.RF6==0) { ; }
      while (PORTFbits.RF6==1) { ; }    
    }
    while (PORTFbits.RF6==0) { ; }      // parity
    while (PORTFbits.RF6==1) { ; }      // stop bit
    while (PORTFbits.RF6==0) { ; }      // stop bit
    mouse_data[p]=val;
  }
  IFS0bits.INT0IF=0;                    // clear INT0 interrupt flag
}

//===============================================================      

void kbd_tx(unsigned x) {
  unsigned i, parity;

  PORTFbits.RF5=1;                      // RF5=1
  PORTFbits.RF6=1;                      // RF6=1
  TRISFbits.TRISF5=0;                   // RF5 is output (kbd data)
  TRISFbits.TRISF6=0;                   // RF6 is output (kbd clock)
  PORTFbits.RF6=0;                      // clock=0
  delay10us(10);                        // delay 100us
  PORTFbits.RF5=0;                      // data=0
  delay10us(1);                         // delay 10us
  TRISFbits.TRISF6=1;                   // RF6 is input 
  delay10us(1);                         // delay 10us
  while (PORTFbits.RF6==1) { ; }        // clock=1 data=0
  while (PORTFbits.RF6==0) { ; }        // clock=0 data=0
  parity=0;
  for (i=0; i<8; i=i+1) {
    PORTFbits.RF5=0x0001&x;
    x=x>>1;
    parity=parity+PORTFbits.RF5;
    while (PORTFbits.RF6==1) { ; }  
    while (PORTFbits.RF6==0) { ; }  
  }
  PORTFbits.RF5=(parity+1)%2;           // odd parity
  while (PORTFbits.RF6==1) { ; }        // clock=1 data=parity
  while (PORTFbits.RF6==0) { ; }        // clock=0 data=parity
  TRISFbits.TRISF5=1;                   // RF5 is input (kbd data)
  while (PORTFbits.RF6==1) { ; }        // clock=1 data=1
  while (PORTFbits.RF6==0) { ; }        // clock=0 data=1
  while (PORTFbits.RF6==1) { ; }        // clock=1 data=ack=0
  while (PORTFbits.RF6==0) { ; }        // clock=0 data=ack=0
  delay_ms(10);                         // ignore ack from keyboard
}

//===============================================================      

int main(void) {
  unsigned i, j;       
  char h[16]="0123456789ABCDEF";

  TRISCbits.TRISC13=0;                  // RC13 is output (LED)

  U1MODE=0x8000;                        // use U1TX 
  U1BRG =95;                            // 19200bps (29.48MHz/16/19200-1=95)
  U1STA =0x0400;                        // start transmission (UTXEN=1)

  TRISFbits.TRISF5 =1;                  // RF5 is input (kbd data)
  TRISFbits.TRISF6 =1;                  // INT0 (RF6) is input (kbd clock)

  INTCON1=0x0000;                       // nothing to set values
  INTCON2bits.ALTIVT=1;                 // use AIVT (Alternate IVT)
  INTCON2bits.INT0EP=1;                 // set INT0 porality (negative edge)
  IFS1=0x0000;                          // clear all interrupt  

  //ADPCFG=0xffff;                      // disable all A/D input pin

  // LCD unit initialize
  TRISB=0xffc0;                         // RB0-RB5 is output
  PORTB=0x0000;                         // RB0-RB5 clear
  delay_ms(40);                         // 40ms delay
  lcd_tx4bit(0, 0b0011); 
  delay_ms(5);        
  lcd_tx4bit(0, 0b0011); 
  delay_ms(1);       
  lcd_tx4bit(0, 0b0011);  
  delay_ms(1);        
  lcd_tx4bit(0, 0b0010);  
  delay_ms(1); 
  lcd_tx4bit(0, 0b0010); 
  lcd_tx4bit(0, 0b1000); 
  lcd_tx4bit(0, 0b0000); 
  lcd_tx4bit(0, 0b1000);
  lcd_tx4bit(0, 0b0000);
  lcd_tx4bit(0, 0b0001);  
  delay_ms(2);
  lcd_tx4bit(0, 0b0000); 
  lcd_tx4bit(0, 0b0110); 

  // display on, cursor off, blink on
  lcd_tx4bit(0, 0b0000); 
  lcd_tx4bit(0, 0b1101); 
  delay_ms(1000);

  kbd_tx(0xf4);                         // enable data reporting
  kbd_tx(0xf3);                         // set sample rate
  delay_ms(10);                         // ignore mouse acknowledge 
  kbd_tx(10);                           // rate = 10sample/sec
  delay_ms(10);                         // ignore mouse acknowledge 

  // enable mouse read interrupt
  IFS0bits.INT0IF=0;                    // clear INT0 interrupt flag
  IEC0bits.INT0IE=1;                    // enable INT0  

  while(1) {
    if (mouse_data[0]!=0) {
      lcd_puthex(mouse_data[0]);        // mouse button, movement sign bit
      //lcd_puthex(mouse_data[1]);      // mouse x movement
      //lcd_puthex(mouse_data[2]);      // mouse y movement
      while (U1STAbits.TRMT==0) { ; }   // while TX buffer is not empty
      U1TXREG=h[mouse_data[0]/16];   
      U1TXREG=h[mouse_data[0]%16];
      U1TXREG=h[mouse_data[1]/16];       
      U1TXREG=h[mouse_data[1]%16];
      while (U1STAbits.TRMT==0) { ; }   // while TX buffer is not empty
      U1TXREG=h[mouse_data[2]/16];      
      U1TXREG=h[mouse_data[2]%16];
      U1TXREG=0x0d;
      U1TXREG=0x0a;
      mouse_data[0]=0;
    }
  }

}
